@@ -1,5 +1,7 @@
 """Support for Vera switches."""
-from typing import Any, Callable, List, Optional
+from __future__ import annotations
+
+from typing import Any
 
 import pyvera as veraApi
 
@@ -10,7 +12,7 @@ from homeassistant.components.switch import (
 )
 from homeassistant.config_entries import ConfigEntry
 from homeassistant.core import HomeAssistant
-from homeassistant.helpers.entity import Entity
+from homeassistant.helpers.entity_platform import AddEntitiesCallback
 from homeassistant.util import convert
 
 from . import VeraDevice
@@ -20,7 +22,7 @@ from .common import ControllerData, get_controller_data
 async def async_setup_entry(
     hass: HomeAssistant,
     entry: ConfigEntry,
-    async_add_entities: Callable[[List[Entity], bool], None],
+    async_add_entities: AddEntitiesCallback,
 ) -> None:
     """Set up the sensor config entry."""
     controller_data = get_controller_data(hass, entry)
@@ -38,7 +40,7 @@ class VeraSwitch(VeraDevice[veraApi.VeraSwitch], SwitchEntity):
 
     def __init__(
         self, vera_device: veraApi.VeraSwitch, controller_data: ControllerData
-    ):
+    ) -> None:
         """Initialize the Vera device."""
         self._state = False
         VeraDevice.__init__(self, vera_device, controller_data)
@@ -57,7 +59,7 @@ class VeraSwitch(VeraDevice[veraApi.VeraSwitch], SwitchEntity):
         self.schedule_update_ha_state()
 
     @property
-    def current_power_w(self) -> Optional[float]:
+    def current_power_w(self) -> float | None:
         """Return the current power usage in W."""
         power = self.vera_device.power
         if power:
