@@ -2,12 +2,11 @@
 from __future__ import annotations
 
 import asyncio
-from collections.abc import Awaitable, Generator, Iterable
+from collections.abc import Awaitable, Callable, Generator, Iterable
 import contextlib
 import logging.handlers
 from timeit import default_timer as timer
 from types import ModuleType
-from typing import Callable
 
 from homeassistant import config as conf_util, core, loader, requirements
 from homeassistant.config import async_notify_setup_error
@@ -16,10 +15,13 @@ from homeassistant.const import (
     EVENT_HOMEASSISTANT_START,
     PLATFORM_FORMAT,
 )
+from homeassistant.core import CALLBACK_TYPE
 from homeassistant.exceptions import HomeAssistantError
 from homeassistant.helpers.typing import ConfigType
 from homeassistant.util import dt as dt_util, ensure_unique_string
 
+# mypy: disallow-any-generics
+
 _LOGGER = logging.getLogger(__name__)
 
 ATTR_COMPONENT = "component"
@@ -39,14 +41,17 @@ BASE_PLATFORMS = {
     "lock",
     "media_player",
     "notify",
+    "number",
     "remote",
     "scene",
     "select",
     "sensor",
+    "siren",
     "switch",
     "tts",
     "vacuum",
     "water_heater",
+    "weather",
 }
 
 DATA_SETUP_DONE = "setup_done"
@@ -280,10 +285,10 @@ async def _async_setup_component(
         await hass.config_entries.flow.async_wait_init_flow_finish(domain)
 
         await asyncio.gather(
-            *[
+            *(
                 entry.async_setup(hass, integration=integration)
                 for entry in hass.config_entries.async_entries(domain)
-            ]
+            )
         )
 
         hass.config.components.add(domain)
@@ -360,9 +365,7 @@ async def async_process_deps_reqs(
 
     Module is a Python module of either a component or platform.
     """
-    processed = hass.data.get(DATA_DEPS_REQS)
-
-    if processed is None:
+    if (processed := hass.data.get(DATA_DEPS_REQS)) is None:
         processed = hass.data[DATA_DEPS_REQS] = set()
     elif integration.domain in processed:
         return
@@ -419,7 +422,7 @@ def _async_when_setup(
         hass.async_create_task(when_setup())
         return
 
-    listeners: list[Callable] = []
+    listeners: list[CALLBACK_TYPE] = []
 
     async def _matched_event(event: core.Event) -> None:
         """Call the callback when we matched an event."""
@@ -440,7 +443,7 @@ def _async_when_setup(
 
 
 @core.callback
-def async_get_loaded_integrations(hass: core.HomeAssistant) -> set:
+def async_get_loaded_integrations(hass: core.HomeAssistant) -> set[str]:
     """Return the complete list of loaded integrations."""
     integrations = set()
     for component in hass.config.components:
@@ -454,7 +457,9 @@ def async_get_loaded_integrations(hass: core.HomeAssistant) -> set:
 
 
 @contextlib.contextmanager
-def async_start_setup(hass: core.HomeAssistant, components: Iterable) -> Generator:
+def async_start_setup(
+    hass: core.HomeAssistant, components: Iterable[str]
+) -> Generator[None, None, None]:
     """Keep track of when setup starts and finishes."""
     setup_started = hass.data.setdefault(DATA_SETUP_STARTED, {})
     started = dt_util.utcnow()
